extend("fiskheroes:hero_basic");
loadTextures({
    "layer1": "smp:rainbow_layer1",
    "layer2": "smp:rainbow_layer2",
    "lights_layer1": "smp:rainbow_light_layer1",
    "lights_layer2": "smp:rainbow_light_layer2",
    "backpack": "smp:defender_backpack",
    "suit": "smp:defender.tx.json",
    "lights_suit": "smp:defender_lights.tx.json",
    "mask": "smp:defender_mask.tx.json",
    "mask_lights": "smp:defender_mask_lights.tx.json",
    "shield": "smp:defender_shield",
    "shield_lights": "smp:defender_shield_lights",
});

var utils = implement("fiskheroes:external/utils");
var energy_projection_backpack = implement("fiskheroes:external/mk85_backpack");
var boosters = implement("smp:external/redstone_mech_boosters");

var shield;
var booster;
var backpack;

function init(renderer) {
    parent.init(renderer);
    renderer.setTexture((entity, renderLayer) => {
        if (entity.getData("fiskheroes:mask_open_timer2") > 0) {
            return "mask";
        }
        return entity.getData("fiskheroes:suit_open_timer") > 0 ? "suit" : renderLayer == "LEGGINGS" ? "layer2" : "layer1";
    });
    renderer.setLights((entity, renderLayer) => {
        if (entity.getData("fiskheroes:mask_open_timer2") > 0) {
            return "mask_lights";
        }
        return entity.getData('fiskheroes:suit_open_timer') > 0 ? "lights_suit" : renderLayer == "LEGGINGS" ? "lights_layer2" : "lights_layer1";
    });
}


function initEffects(renderer) {
    boosters = boosters.create(renderer, "fiskheroes:deep_blue_fire_layer_%s", true);
    backpack = energy_projection_backpack.create(renderer, "backpack");

    utils.bindBeam(renderer, "fiskheroes:charged_beam", "fiskheroes:mysterio_beam", "body", 0xA63534, [
        { "offset": [6.75, 10.0, 3.0], "size": [2.0, 2.0] },
        { "offset": [10.0, 0.5, 3.0], "size": [2.0, 2.0] },
        { "offset": [6.5, -4.5, 3.0], "size": [2.0, 2.0] },
        { "offset": [-6.75, 10.0, 3.0], "size": [2.0, 2.0] },
        { "offset": [-10.0, 0.5, 3.0], "size": [2.0, 2.0] },
        { "offset": [-6.5, -4.5, 3.0], "size": [2.0, 2.0] }
    ]).setParticles(renderer.createResource("PARTICLE_EMITTER", "fiskheroes:impact_charged_beam"));


    utils.bindParticles(renderer, "smp:rainbow_defender").setCondition(entity => entity.getData("fiskheroes:flying"));

    shield = renderer.createEffect("fiskheroes:shield");
    shield.texture.set("shield", "shield_lights");
    shield.anchor.set("rightArm");
    shield.setRotation(0.0, 0.0, -10.0).setCurve(15.0, 50.0);
    shield.large = true;
}

function initAnimations(renderer) {
    parent.initAnimations(renderer);

    addAnimation(renderer, "basic.CHARGED_BEAM", "fiskheroes:dual_aiming").setData((entity, data) => data.load(Math.max(entity.getInterpolatedData("fiskheroes:beam_charge") * 5 - 4, 0)));

    utils.addFlightAnimationWithLanding(renderer, "rainbow.FLIGHT", "fiskheroes:flight/iron_man.anim.json");
    utils.addHoverAnimation(renderer, "rainbow.HOVER", "fiskheroes:flight/idle/iron_man");
    addAnimationWithData(renderer, "rainbow.LAND", "fiskheroes:superhero_landing", "fiskheroes:dyn/superhero_landing_timer")
        .priority = -8;

    utils.addAnimationEvent(renderer, "FLIGHT_DIVE", "fiskheroes:iron_man_dive");
    utils.addAnimationEvent(renderer, "FLIGHT_DIVE_ROLL", "fiskheroes:falcon_dive_roll");
    
    addAnimationWithData(renderer, "rainbow.ROLL", "fiskheroes:flight/barrel_roll", "fiskheroes:barrel_roll_timer")
            .priority = 10;
}

function render(entity, renderLayer, isFirstPersonArm) {
    boosters.render(entity, renderLayer, isFirstPersonArm, false);
    
    backpack.render(entity.getInterpolatedData("fiskheroes:beam_charge"));

    shield.unfold = entity.getInterpolatedData("fiskheroes:shield_timer");
    shield.setOffset(2.9 + 1.8 * Math.min(shield.unfold * 5, 1), 6.0, 0.0);
    shield.render();
}